#!/usr/bin/env python
###############################################################################
# (c) Copyright 2018 CERN                                                     #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################

import logging
import os
from datetime import datetime


def main():
    from argparse import ArgumentParser

    parser = ArgumentParser()
    parser.add_argument("--token", default=os.environ.get("GITLAB_TOKEN"))

    args = parser.parse_args()
    logging.basicConfig(level=logging.DEBUG)

    try:
        import gitlab
    except ImportError:
        parser.error("install python-gitlab to be able to use this script")

    if not args.token:
        parser.error("either set GITLAB_TOKEN or pass the --token option")

    server = gitlab.Gitlab("https://gitlab.cern.ch/", args.token)

    logging.debug("retrieve group lhcb-datapkg")
    group = server.groups.get("lhcb-datapkg")
    logging.debug("listing projects in %s", group.name)
    packages = {p.path for p in group.projects.list(all=True)}

    logging.debug("looping over subgroups")
    for subgroup in group.subgroups.list(all=True):
        logging.debug("retrieve subgroup %s", subgroup.name)
        g = server.groups.get(subgroup.id)

        logging.debug("listing projects in %s", g.name)
        packages.update(f"{g.path}/{p.path}" for p in g.projects.list(all=True))

    print("# List of known package names in gitlab")
    print(f"# generated at {datetime.now()} by {parser.prog}")
    print("\n".join(sorted(packages)))


if __name__ == "__main__":
    main()



#!/usr/bin/env python
###############################################################################
# (c) Copyright 2018 CERN                                                     #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################
"""
Generate and print the environment banner
"""


def _center(txt, width):
    """
    Return a string of length `width` with the string `txt` centered

    >>> _center('abc', 14)
    '     abc      '
    """
    left = " " * int((width - len(txt)) / 2)
    right = " " * (width - len(txt) - len(left))
    return f"{left}{txt}{right}"


def generate(width=80, env=None):
    """
    Return the environment banner string.
    """
    from os import environ, pathsep

    if env is None:
        env = environ

    lines = []
    lines.append("*" * width)
    lines.append("*{}*".format(_center("---- LbEnv ----", width - 2)))
    if "BINARY_TAG" in env:
        lines.append(
            "*{}*".format(
                _center(
                    "using platform id {}".format(env["BINARY_TAG"]),
                    width - 2,
                )
            )
        )

    lines.append("*" * width)
    if "User_release_area" in env:
        lines.append(
            " --- User_release_area is set to {}".format(env["User_release_area"])
        )
    if env.get("CMAKE_PREFIX_PATH"):
        lines.append(" --- CMAKE_PREFIX_PATH is set to:")
        lines.extend(f"    {item}" for item in env["CMAKE_PREFIX_PATH"].split(pathsep))
    else:
        lines.append(" --- CMAKE_PREFIX_PATH is NOT set!")
    lines.append("-" * width)
    return "\n".join(lines)


if __name__ == "__main__":
    print(generate())



###############################################################################
# (c) Copyright 2018 CERN                                                     #
#                                                                             #
# This software is distributed under the terms of the GNU General Public      #
# Licence version 3 (GPL Version 3), copied verbatim in the file "LICENSE".   #
#                                                                             #
# In applying this licence, CERN does not waive the privileges and immunities #
# granted to it by virtue of its status as an Intergovernmental Organization  #
# or submit itself to any jurisdiction.                                       #
###############################################################################

__author__ = "Marco Clemencic <marco.clemencic@cern.ch>"

import os
import sys

assert sys.version_info >= (2, 6), "Python >= 2.6 is required"


def _cmakeStr(txt):
    return " ".join(
        '"{}"'.format(entry.replace('"', '\\"')) for entry in str(txt).split(os.pathsep)
    )


_CMTPROJECTPATH_FMT_SH = (
    'export CMTPROJECTPATH="{0}' '${{CMTPROJECTPATH:+:${{CMTPROJECTPATH}}}}"\n'
)
_CMTPROJECTPATH_FMTS = {
    "csh": """if ( $?CMTPROJECTPATH ) then
  setenv CMTPROJECTPATH "{0}:${{CMTPROJECTPATH}}"
else
  setenv CMTPROJECTPATH "{0}"
endif
""",
    "sh": _CMTPROJECTPATH_FMT_SH,
}


class SearchPathEntry:
    def __init__(self, path):
        self.path = path

    def __str__(self):
        return str(self.path)

    def toCMake(self):
        return f"list(INSERT CMAKE_PREFIX_PATH 0 {_cmakeStr(self)})\n"

    def toCMT(self, shell="sh"):
        # FIXME: should we escape quotes?
        return _CMTPROJECTPATH_FMTS[shell].format(self)

    def __repr__(self):
        return f"{self.__class__.__name__}({self.path!r})"


class EnvSearchPathEntry(SearchPathEntry):
    def __init__(self, envname, default=None):
        if default is None and envname not in os.environ:
            raise ValueError(f"environment variable {envname} not defined")
        self.envname = envname
        self.default = default

    def __str__(self):
        return str(os.environ.get(self.envname, self.default))

    def toCMake(self):
        return (
            f"if(NOT ENV{{{self.envname}}})\n"
            "  # use a default value\n"
            f"  set(ENV{{{self.envname}}} {_cmakeStr(self)})\n"
            "endif()\n"
            f'list(INSERT CMAKE_PREFIX_PATH 0 "$ENV{{{self.envname}}}")\n'
        )

    def toCMT(self, shell="sh"):
        if self.envname == "CMAKE_PREFIX_PATH":
            return ""  # ignore CMAKE_PREFIX_PATH for CMT builds
        if shell == "csh":
            fmt = (
                "if ( ! $?{0} ) then\n"
                "  # use a default value\n"
                '  setenv {0} "{1}"\n'
                "endif\n"
            )
        else:
            fmt = (
                'if [ -z "${0}" ] ; then\n'
                "  # use a default value\n"
                '  export {0}="{1}"\n'
                "fi\n"
            )
        # FIXME: should we escape quotes?
        return fmt.format(self.envname, self) + _CMTPROJECTPATH_FMTS[shell].format(
            "${" + self.envname + "}"
        )

    def __repr__(self):
        return f"{self.__class__.__name__}({self.envname!r}, {str(self)!r})"


def _toEntry(entry):
    """ensure that the entry is a SearchPathEntry"""
    if isinstance(entry, SearchPathEntry):
        return entry
    return SearchPathEntry(entry)


class SearchPath:
    def __init__(self, entries):
        self.entries = list(entries)

    def __iter__(self):
        from itertools import chain

        return filter(
            None,
            chain.from_iterable(str(entry).split(os.pathsep) for entry in self.entries),
        )

    def insert(self, index, value):
        return self.entries.insert(index, value)

    def append(self, value):
        return self.entries.append(value)

    def extend(self, values):
        return self.entries.extend(values)

    def __setitem__(self, index, value):
        if isinstance(value, SearchPath):
            value = value.entries
        return self.entries.__setitem__(index, value)

    def __getitem__(self, index):
        if isinstance(index, slice):
            return self.__class__(self.entries[index])
        else:
            return self.entries[index]

    def __str__(self):
        return os.pathsep.join(self)

    def __nonzero__(self):
        return bool(self.entries)

    def __add__(self, other):
        if isinstance(other, SearchPath):
            other = other.entries
        return SearchPath(self.entries + other)

    def toCMake(self):
        # we need to add them in reverse order because the 'prepend'
        return "\n".join(_toEntry(entry).toCMake() for entry in self.entries[::-1])

    def toCMT(self, shell="sh"):
        # we need to add them in reverse order because the 'prepend'
        return "\n".join(_toEntry(entry).toCMT(shell) for entry in self.entries[::-1])

    def __repr__(self):
        return f"{self.__class__.__name__}({self.entries!r})"

    def __len__(self):
        return len(self.entries)


def _defaultPath():
    """
    Return the default search path, based on the variables CMAKE_PREFIX_PATH
    and CMTPROJECTPATH.
    """
    env_vars = ["CMAKE_PREFIX_PATH", "CMTPROJECTPATH"]
    path = []
    for name in env_vars:
        try:
            path.append(EnvSearchPathEntry(name, ""))
        except ValueError:
            pass  # ignore undefined variables
    path.append(
        EnvSearchPathEntry("LHCBPROJECTPATH", default="/cvmfs/lhcb.cern.ch/lib/lhcb")
    )
    return path


path = SearchPath(_defaultPath())


class Error(RuntimeError):
    """
    Base class for LbRun exceptions.
    """
